library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity slizer is
	port
	(
	PPM_L	  : in  std_logic_vector (20 downto 0);		-- Left LED data input
	PPM_R	  : in  std_logic_vector (20 downto 0);		-- Right LED data input
	AUX	  : in  std_logic_vector (5 downto 0);			-- misc flags, LEDs etc
	START	  : in 	std_logic;									-- invokes a serial transmission
	nRESET  : in  std_logic;									-- global reset
	CLKin   : in  std_Logic;
	CLK33	  : out std_logic;									-- clock lo-hi transition clocks valid data.  
	DATA33  : out std_logic;
	STB33	  : out std_logic 									-- hi pulse latches and enables new data
	);
-- The '33' suffix indicates these are 3.3V logic signals which require external buffering
end slizer;

architecture slizer_arch of slizer is
	signal	slotcounter	:integer range 0 to 50;
	signal	shiftreg	:std_logic_vector (47 downto 0);
	signal  in_vector   :std_logic_vector (47 downto 0); 

begin
-- define parallel load data i.t.o. inputs
	in_vector(20 downto 0) <= PPM_L(20 downto 0);
	in_vector(41 downto 21)<= PPM_R(20 downto 0);
	in_vector(47 downto 42)<= AUX(5 downto 0);	

-- set up bitcounter as a quasi state machine	
process (CLKin, nRESET) begin
	if nRESET='0' then
		slotcounter <= 0;
	elsif CLKin'event and CLKin='1' then
		if slotcounter=0 and START='0' then
			slotcounter <= 0;								-- wait in zero state for START...
		elsif slotcounter = 49 then
			slotcounter <= 0;								-- ...return to zero after all is done
		else
			slotcounter <= slotcounter+1;				-- ... otherwise count
		end if;
	end if;
end process;

-- set up shift register to load input data and then shift out MSB first.
process (CLKin,nRESET) begin
	if nRESET='0' then
		shiftreg <= X"000000000000";
	elsif CLKin'event and CLKin='1' then
		if slotcounter=0 then					-- parallel load on slot 0
			shiftreg <= in_vector;
		elsif slotcounter < 49 then
			shiftreg(46 downto 0) <= shiftreg(47 downto 1);
			shiftreg(47)<='0';
		end if;
	end if;
end process;

-- define clock and data signals
DATA33 <= shiftreg(0);
CLK33  <= (not CLKin) when (slotcounter > 0 and slotcounter < 49) else '0';
STB33  <= '1' when slotcounter=49 else '0';
		
end slizer_arch;

