-- This module generates the clock enables REGCLKEN and AVGCLKEN during the 3rd CLK64fs period
-- after LRCLKfs (hi), ie in R ch sample. It also generates the MUXCLK of fs/32 
-- for the LED group multiplexor
-- *** Rev B 21/10/2001 DECAYCLKEN has been removed
 
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity sysgen is
	port
	(
	CLK64fs: in STD_LOGIC;
	RESETL: in STD_LOGIC;
	LSLOT: in STD_LOGIC_VECTOR (4 downto 0) ;
	RSLOT: in STD_LOGIC_VECTOR (4 downto 0) ;
	INCLKEN_L: out STD_LOGIC;
	INCLKEN_R: out STD_LOGIC;
	PKCLKEN_L: out STD_LOGIC;
	PKCLKEN_R: out STD_LOGIC;
	AVGCLKEN:	out STD_LOGIC;
	UPDATELEDS: out STD_LOGIC
	);
end sysgen;

architecture sysgen_arch of sysgen is
	signal	ClockCntr	: integer range 0 to 511;
	
begin
	
-- implement the divide by 2^10 counter to define:
-- This counter is clocked by CLK64fs and enabled at wordframe clock rate 
-- during LBitslot1. Output changes on transition to Lbitslot2
-- *** AVGCLKEN clock enable signal at 86.12Hz (44.1kHz/2^9) for VU meter averaging
---*** UPDATELEDS strobe at 2756Hz to invoke serial transmission of LED data  	
	process (CLK64fs, RESETL) begin
		if RESETL='0' then 
			ClockCntr <= 0;
		elsif CLK64fs'event and CLK64fs='1' then
			if (LSLOT = 1) then
				if ClockCntr = 511 then
					ClockCntr <= 0;
				else
					ClockCntr <= ClockCntr + 1;
				end if;
			end if;	
		end if;
	end process;

-- assign outputs for INCLKEN_L,R asserted during L,R bitslot 28
-- assign outputs for PKCLKEN_L,R asserted during L,R bitslot 30
-- assign output for AVGCLKEN
	INCLKEN_L <= '1' when LSLOT = 28 else '0';
	PKCLKEN_L <= '1' when LSLOT = 30 else '0';
	INCLKEN_R <= '1' when RSLOT = 28 else '0';
	PKCLKEN_R <= '1' when RSLOT = 30 else '0';					
	AVGCLKEN <= '1' when ClockCntr = 8 and (LSLOT=30 or RSLOT=30) else '0';			
	UPDATELEDS <= '1' when ClockCntr mod 8 = 1 and LSLOT=26 else '0';		

end sysgen_arch;




